﻿--
-- Allrounder
-- specialization for Allrounder, to control the swap bodies from allrounder. 
-- New animation values, when near enough to swap body, higher rpm when lift, 
-- real attacher, only attach a body when drive with lowered chassis under a body and then lift.
-- 
--
-- @author Bluebaby210 www.mod-portal.de
-- @date 31.07.2013
--
-- @version: v 1.0 release at modcontest 2013
--
--

Allrounder = {};

function Allrounder.prerequisitesPresent(specializations)
	Vehicle.registerJointType("fastCoupling");
	print('register JointType '..'"'..'fastCoupling'..'"' )
    return true;
end;

function Allrounder:load(xmlFile)

	self.detachBody = SpecializationUtil.callSpecializationsFunction("detachBody");
	self.lowerAllrounder = SpecializationUtil.callSpecializationsFunction("lowerAllrounder");
	self.attacherMoving = SpecializationUtil.callSpecializationsFunction("attacherMoving");
	self.capacityChange = SpecializationUtil.callSpecializationsFunction("capacityChange");
	self.newAnimationValues = SpecializationUtil.callSpecializationsFunction("newAnimationValues");
	self.rpmAndAttacher = SpecializationUtil.callSpecializationsFunction("rpmAndAttacher");
	
	
	self.attacherJointToTrans = {};
	self.attacherJointToTrans.index = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherMove#partindex"));
		if self.attacherJointToTrans.index ~= nil then
			local x,y,z = getTranslation(self.attacherJointToTrans.index);
			self.attacherJointToTrans.startX = x;
			self.attacherJointToTrans.startY = y;
			self.attacherJointToTrans.startZ = z;
			self.attacherJointToTrans.movedY = (y - 10);
		end;
	
	self.attacherSearchPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherSearchPoint#index"));
	
	self.vehicleAttach = nil;
	self.implementAttached =  nil;
	self.searchImplement = true;
	self.firstRun = true;
	self.attacherMoved = true;
	self.attacherMovedState = 0;
	self.liftVehicle = false;
	self.firstLower = false;
	self.startTimer = false;
	self.startTimerLower = false;
	self.timerLift = 0;
	self.timerDanger = 0;
	self.wasToFastNew = false;
	self.toFastLower = false;
	self.toFastLower2 = false;
	self.newSchemaOverlay = {};
	self.newSchemaOverlay.overlay = self.schemaOverlay.overlay;
	self.newSchemaOverlay.overlaySelected = self.schemaOverlay.overlaySelected;
	self.loadSavegame = false;

	self.moreRpm = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.allrounderTrigger#moreRpm"), true);
	self.realAttacher = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.allrounderTrigger#realAttacher"), true);

	local triggerAllrounder = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.allrounderTrigger#index"));
	if triggerAllrounder ~= nil then
		self.playerTrigger = Allrounder.playerTrigger;
		self.allrounderTrigger = triggerAllrounder;
		addTrigger(self.allrounderTrigger, "playerTrigger", self);
		self.playerInTrigger = false;
	end;
	
	
	self.distanceLargeEnough = false;
	self.distanceSmallEnough = true;
	--	<newAnimationSupport>-- to change the values in Animations for "foldable" in Game, with new values. To set higher endTrans for example.
	self.backupAnimations = {};
	self.newAnimations = {};
	self.animationNames = {};
	

	self.distancesForNewValues = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.newAnimationSupport#distance"), 6);
	self.distance = (self.distancesForNewValues + 1);

	local i=0;
		while true do
			local key = string.format("vehicle.newAnimationSupport.animation(%d)", i);
				if not hasXMLProperty(xmlFile, key) then
					break;
				end;
				local name = getXMLString(xmlFile, key.."#name");
				if name ~= nil then
					local newAnimation = {};
					newAnimation.name = name;
					newAnimation.newParts = {};
					table.insert(self.animationNames, name);
					local partI = 0;
					while true do
						local partKey = key..string.format(".partOld(%d)", partI);
						if not hasXMLProperty(xmlFile, partKey) then
							break;
						end;
	
						local line = Utils.getNoNil(getXMLInt(xmlFile, partKey.."#line"), 0);
						local startRot = Utils.getRadiansFromString(getXMLString(xmlFile, partKey.."#newStartRot"), 3);
						local endRot = Utils.getRadiansFromString(getXMLString(xmlFile, partKey.."#newEndRot"), 3);
						local startTrans = Utils.getVectorNFromString(getXMLString(xmlFile, partKey.."#newStartTrans"), 3);
						local endTrans = Utils.getVectorNFromString(getXMLString(xmlFile, partKey.."#newEndTrans"), 3);
						local startScale = Utils.getVectorNFromString(getXMLString(xmlFile, partKey.."#newStartScale"), 3);
						local endScale = Utils.getVectorNFromString(getXMLString(xmlFile, partKey.."#newEndScale"), 3);
						local startTime = getXMLFloat(xmlFile, partKey.."#startTime");
						local duration = getXMLFloat(xmlFile, partKey.."#duration");
						local endTime = getXMLFloat(xmlFile, partKey.."#endTime");

						if self.animations[newAnimation.name].parts[line] ~= nil then
							if endTime ~= nil then
								duration = endTime - startTime;
							end;

						local newPart = {};
						newPart.startTime = startTime*1000;
						newPart.duration = duration*1000;

 							if startRot ~= nil then
								newPart.startRot = startRot;
							end;
 							if endRot ~= nil then
								newPart.endRot = endRot;
							end;
 							if startTrans ~= nil then
								newPart.startTrans = startTrans;
							end;
 							if endTrans ~= nil then
								newPart.endTrans = endTrans;
							end;
							if startScale ~= nil then
								newPart.startScale = startScale;
							end;
							if endScale ~= nil then
								newPart.endScale = endScale;
							end;
							table.insert(newAnimation.newParts, newPart);
 						end;
				local newValues = {};
				newValues.value = {};
				local newValue = {};
				newValue.startRotPos = {}
				newValue.endRotPos = {};
				newValue.startTransPos = {}
				newValue.endTransPos = {};
				newValue.startScalePos = {}
				newValue.endScalePos = {};
				table.insert(newValues.value, newValue);
				self.backupAnimations[name] = newValues;
				
 					partI = partI + 1;
 				end;
				newAnimation.newPartsReverse = {};
				for _, newPart in ipairs(newAnimation.newParts) do
					table.insert(newAnimation.newPartsReverse, newPart);
				end;
				
				
				table.sort(newAnimation.newParts, AnimatedVehicle.animPartSorter);
				table.sort(newAnimation.newPartsReverse, AnimatedVehicle.animPartSorterReverse);
				
					self.newAnimations[name] = newAnimation;
				end;
	
			i = i+1;
		end;
		
	for i=1, table.getn(self.animationNames) do
		for k=1, table.getn(self.animations[self.animationNames[i]].parts) do
			self.backupAnimations[self.animationNames[i]].value[k] = self.animations[self.animationNames[i]].parts[k];
			self.backupAnimations[self.animationNames[i]].value[k].startRotPos = self.animations[self.animationNames[i]].parts[k].startRot;
			self.backupAnimations[self.animationNames[i]].value[k].endRotPos = self.animations[self.animationNames[i]].parts[k].endRot;
			self.backupAnimations[self.animationNames[i]].value[k].startTransPos = self.animations[self.animationNames[i]].parts[k].startTrans
			self.backupAnimations[self.animationNames[i]].value[k].endTransPos = self.animations[self.animationNames[i]].parts[k].endTrans;
			self.backupAnimations[self.animationNames[i]].value[k].startScalePos = self.animations[self.animationNames[i]].parts[k].startScale;
			self.backupAnimations[self.animationNames[i]].value[k].endScalePos = self.animations[self.animationNames[i]].parts[k].endScale;
		end;
	end;

end;

function Allrounder:delete()
	self.newSchemaOverlay = nil;
    removeTrigger(self.allrounderTrigger);
end;

function Allrounder:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Allrounder:keyEvent(unicode, sym, modifier, isDown)
end;

function Allrounder:readStream(streamId, connection)
	self.distanceSmallEnough = streamReadBool(streamId);
	self.distanceLargeEnough = streamReadBool(streamId);
	self.searchImplement = streamReadBool(streamId);
	self.loadSavegame = streamReadBool(streamId);
	self.attacherMoved = streamReadBool(streamId);
	self.attacherMovedState = streamReadInt8(streamId);
	self.firstRun = streamReadBool(streamId);
	self.capacity = streamReadFloat32(streamId);
	self.moreRpm = streamReadBool(streamId);
	self.realAttacher = streamReadBool(streamId);
	
	if self.distanceLargeEnough and  self.distanceSmallEnough == false then
		for i=1, table.getn(self.animationNames) do
			self.names = self.animationNames[i];
			for k=1, table.getn(self.newAnimations[self.names].newParts) do
				if self.newAnimations[self.names].newParts[k].startRot ~= nil then
					self.animations[self.names].parts[k].startRot = self.newAnimations[self.names].newParts[k].startRot;
				end;
				if self.newAnimations[self.names].newParts[k].endRot ~= nil then
					self.animations[self.names].parts[k].endRot = self.newAnimations[self.names].newParts[k].endRot;
				end;
				if self.newAnimations[self.names].newParts[k].startTrans ~= nil then
					self.animations[self.names].parts[k].startTrans = self.newAnimations[self.names].newParts[k].startTrans;
				end;
				if self.newAnimations[self.names].newParts[k].endTrans ~= nil then
					self.animations[self.names].parts[k].endTrans = self.newAnimations[self.names].newParts[k].endTrans;
				end;
				if self.newAnimations[self.names].newParts[k].startScale ~= nil then
					self.animations[self.names].parts[k].startScale = self.newAnimations[self.names].newParts[k].startScale;
				end;
				if self.newAnimations[self.names].newParts[k].endScale ~= nil then
					self.animations[self.names].parts[k].endScale = self.newAnimations[self.names].newParts[k].endScale;
				end;
			end;
		end;
	end;
	
	if self.searchImplement == false then
		if self.vehicleAttach ~= nil then
			if table.getn(self.attachedImplements) == 1 then
				for k,v in pairs(self.attachedImplements) do
					if v.object.detachFromAllrounder ~= nil then
						self.implementAttached = v.object;
						self.schemaOverlay.overlay = self.implementAttached.newBodySchemaOverlay.overlay;
						self.schemaOverlay.overlaySelected = self.implementAttached.newBodySchemaOverlay.overlaySelected;
					end;
					if v.object:getIsSelectable() then
						v.object.isSelectable = false;
					end;
				end;
			end;
		end;
	end;

end;


function Allrounder:writeStream(streamId, connection)
	streamWriteBool(streamId, self.distanceSmallEnough);
	streamWriteBool(streamId, self.distanceLargeEnough);
	streamWriteBool(streamId, self.searchImplement);
	streamWriteBool(streamId, self.loadSavegame);
	streamWriteBool(streamId, self.attacherMoved);
	streamWriteInt8(streamId, self.attacherMovedState);
	streamWriteBool(streamId, self.firstRun);
	streamWriteFloat32(streamId, self.capacity);
	streamWriteBool(streamId, self.moreRpm);
	streamWriteBool(streamId, self.realAttacher);
end;

function Allrounder:readStreamUpdate(streamId, connection)	
end;

function Allrounder:writeStreamUpdate(streamId, connection)
end;

function Allrounder:playerTrigger(triggerId, otherId, onEnter, onLeave, onStay)
	if onEnter then
		if g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
			self.playerInTrigger = true;
		end;
	elseif onLeave then
		if g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
			self.playerInTrigger = false;
		end;
	end;
end;

function Allrounder:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local rpm = getXMLBool(xmlFile, key.."#moreRpmLift");
	local attacher = getXMLBool(xmlFile, key.."#realAttacher");
		if rpm ~= nil then
			self.moreRpm = rpm;
		end;
		if attacher ~= nil then
			self.realAttacher = attacher;
		end;

	return BaseMission.VEHICLE_LOAD_OK;

end;

function Allrounder:getSaveAttributesAndNodes(nodeIdent)
		local attributes = 'moreRpmLift="' .. tostring(self.moreRpm) .. '" realAttacher="'.. tostring(self.realAttacher) .. '"';
		return attributes, nil;
end;

function Allrounder:updateTick(dt)

	if table.getn(self.attachedImplements) == 0 then
		self.distances = {};
			for k,v in pairs(g_currentMission.vehicles) do
					if v ~= self and v ~= self.attacherVehicle and v.attacherVehicle == nil then
						if v.attacherJoint ~= nil and v.attacherJoint.jointType == Vehicle.JOINTTYPE_FASTCOUPLING then
							if v.detachFromAllrounder ~= nil and SpecializationUtil.hasSpecialization(ExchangeableBody, v.specializations) then
								local x,y,z = getWorldTranslation(self.attacherSearchPoint);
								local x1,y1,z1 = getWorldTranslation(v.attacherJoint.node);
								local distance = Utils.vector3Length(x-x1, y-y1, z-z1);
								distance = math.floor(distance*100);
								table.insert (self.distances, (distance/100));
								end;
						end;
					end;
			end;
			if table.getn(self.distances) > 0 then
				table.sort(self.distances);
				self.distance = self.distances[1];
					if self.foldAnimTime == 0 then
						if self.distance <= self.distancesForNewValues then
							if self.distanceSmallEnough then
								self:newAnimationValues(true, false);
							end;
							self.firstLower = false;
						elseif self.distance > self.distancesForNewValues then
							if self.distanceLargeEnough then
								self:newAnimationValues(false, true);
							end;
							self.firstLower = false;
						end;
					end;
					
					if self.foldAnimTime == 1 then
						if self.distance <= (self.distancesForNewValues + 0.5) and self.distanceSmallEnough then
							self.firstLower = true;
						else
							self.firstLower = false;
						end;
					end;
			end;
		self.distances = nil;
	end;

	if self.firstRun then
		if table.getn(self.attachedImplements) == 0 then
			if self.distance <= self.distancesForNewValues then
				self:newAnimationValues(true, false);
				self:setFoldState(self.foldAnimTime, false);
			end;
			if self.foldAnimTime == 1 and self.distance > self.distancesForNewValues and self.realAttacher then
				if self.attacherMoved and self.attacherJointToTrans.index ~= nil then
					self.attacherMovedState = 0;
					self:attacherMoving(self.attacherMoved, self.attacherMovedState)
					self.attacherMoved = false;
				end;
			end;
			if self.fillLevel == 0 then
				self.capacity = 0;
				self:setFillLevel(self.fillLevel, self.currentFillType);
			end;
		end;
		if table.getn(self.attachedImplements) == 1 then
			self.distance = 0;
			self:newAnimationValues(true, false);
			self:setFoldState(self.foldAnimTime, false);
			for k,v in pairs(self.attachedImplements) do
				if v.object.fuelFill ~= nil and v.object.liquidManure == nil then
					self.implementAttached = v.object;
					self.capacity = self.implementAttached.capacity;
					self.implementAttached.capacity = 0;
					self.implementAttached.fillLevel = 0;
					self.implementAttached.currentFillType = Fillable.FILLTYPE_UNKNOWN;
				end;
				if v.object.fuelFill == nil then
					self.implementAttached = v.object;
					self.fillLevel = 0;
					self.capacity = 0;
					self:setFillLevel(self.fillLevel, self.currentFillType);
				end;
				if v.object:getIsSelectable() then
					v.object.isSelectable = false;
				end;
			end;
		end;
		self.firstRun = false;
	end;

	if self.vehicleAttach ~= nil then
		if self.implementAttached ~= nil and self:getRealAnimationTime("lift") >= self:getAnimationDuration("lift") then
			local toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 10;
			local toFast2 = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 20;
			self.wasToFast = toFast;
			self.toFastLower = toFast2;
			if self.toFastLower then
				self.startTimerLower = true;
				self:lowerAllrounder();
			end;
		else
			if self.implementAttached ~= nil then
				self.wasToFast = false;
				self.toFastLower = false;
			end;
			if self.implementAttached == nil and self:getRealAnimationTime("lift") >= self:getAnimationDuration("lift") then
				local toFast1 = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 17;
				local toFast2 = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 26;
				self.wasToFastNew = toFast1;
				self.toFastLower2 = toFast2;
				if self.toFastLower2 then
					self.startTimerLower = true;
					self:lowerAllrounder();
				end;
			elseif self.implementAttached == nil then
				self.wasToFastNew = false;
				self.toFastLower2 = false;
			end;	
		end;
	end;

end;

function Allrounder:update(dt)

	if self.searchImplement then
		if self.vehicleAttach ~= nil then
			for k,v in pairs(self.attachedImplements) do
				if v.object:getIsSelectable() then
					v.object.isSelectable = false;
				end;
				if v.object.detachFromAllrounder ~= nil then 
					self.implementAttached = v.object;
					self.schemaOverlay.overlay = self.implementAttached.newBodySchemaOverlay.overlay;
					self.schemaOverlay.overlaySelected = self.implementAttached.newBodySchemaOverlay.overlaySelected;
				end;
				if v.object.fuelFill ~= nil and v.object.liquidManure == nil and self.loadSavegame then
					self.implementAttached = v.object;
					if self.implementAttached.capacity ~= 0 then
						self:capacityChange(true, v.object);
					end;
				end;
			end;
		end;
		self.searchImplement = false;
		self.loadSavegame = true;
	end;
	
	if self:getIsActiveForInput() then
		if table.getn(self.attachedImplements) == 0 then
			if self.foldAnimTime == 0 or (self.foldAnimTime < 1 and self.foldAnimTime >= 0.99) then
				if self.attacherMoved and self.attacherJointToTrans.index ~= nil then
					self.attacherMovedState = 0; 
					self:attacherMoving(self.attacherMoved, self.attacherMovedState)
					self.attacherMoved = false;
				end;
			elseif self.foldAnimTime == 1 then
				if self.realAttacher then
					if self.distanceSmallEnough == false and self.distanceLargeEnough then
						if self.attacherMoved and self.attacherJointToTrans.index ~= nil then
							self.attacherMovedState = 1;
							self:attacherMoving(self.attacherMoved, self.attacherMovedState)
							self.attacherMoved = false;
						end;
					end;
				elseif not self.realAttacher then
					if self.attacherMoved and self.attacherJointToTrans.index ~= nil then
						self.attacherMovedState = 1;
						self:attacherMoving(self.attacherMoved, self.attacherMovedState)
						self.attacherMoved = false;
					end;				
				end;
			end;
			if self:getIsAnimationPlaying("lift") then
				self.attacherMoved = true;
			end;
		end;

		if self.moreRpm then
			if self:getIsAnimationPlaying("lift") then
				if self:getToggledFoldDirection() == -1 then
					if self.vehicleAttach.motor ~= nil then
						self.vehicleAttach.motor.minRpm = math.max(self.vehicleAttach.motor.minRpm-dt*2, -1000);
					elseif self.secondAttacher.motor ~= nil then
						self.secondAttacher.motor.minRpm = math.max(self.secondAttacher.motor.minRpm-dt*2, -1000);
					end;
				end;
			else
				if self.vehicleAttach.motor ~= nil and self.lastRpm ~= 0 then
					self.vehicleAttach.motor.minRpm = math.min(self.vehicleAttach.motor.minRpm+dt*2, self.lastRpm);
				elseif self.secondAttacher.motor ~= nil and self.lastRpm ~= 0 then
					self.secondAttacher.motor.minRpm = math.min(self.secondAttacher.motor.minRpm+dt*2, self.lastRpm);
				end;
			end;
		end;
		
		if self.implementAttached ~= nil then
		
			if self.implementAttached:getIsFoldAllowed() then
				if InputBinding.hasEvent(self.implementAttached.foldInputButton) then
					self.implementAttached:setFoldState(self.implementAttached:getToggledFoldDirection(), false);
				end;
			end;
			
			if self:getRealAnimationTime("lift") >= self:getAnimationDuration("lift") then		
					if InputBinding.hasEvent(InputBinding.DetachBodyOne) then
						self:detachBody();
					end;
			end;
			
			if self:getRealAnimationTime("lift") < self:getAnimationDuration("lift") then
					if InputBinding.hasEvent(InputBinding.DetachBodyOne) then
						self.startTimer = true;
					end;
			end;
			
			if self.implementAttached ~= nil and self.implementAttached.setIsTurnedOn ~= nil then
				if self.implementAttached.allowsSpraying then
					if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
						self.implementAttached:setIsTurnedOn(not self.implementAttached.isTurnedOn);
					end;
				end;
			end;
			
		end;

		
		if self.startTimer then
			self.timerLift = self.timerLift + dt;
			if self.timerLift >= 1 and self.timerLift <= 100 then
				self.liftVehicle = true;
			elseif self.timerLift >= 4500 then
				self.startTimer = false;
				self.liftVehicle = false;
				self.timerLift = 0;
			end;
		end;
		
		if self.startTimerLower then
			self.timerDanger = self.timerDanger + dt;
			if self.timerDanger >= 1 and self.timerDanger <= 100 then
				self.dangerWarning = true;
			elseif self.timerDanger >= 4500 then
				self.startTimerLower = false;
				self.dangerWarning = false;
				self.timerDanger = 0;
			end;
		end;

	end;


	if self.vehicleAttach ~= nil and not self:getIsActive() and self.playerInTrigger then
	
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
			self:rpmAndAttacher(true, false);
		end;
		if self.moreRpm then
			g_currentMission:addHelpButtonText(g_i18n:getText("noMoreRpm"), InputBinding.IMPLEMENT_EXTRA4);		
		elseif not self.moreRpm then
			g_currentMission:addHelpButtonText(g_i18n:getText("moreRpm"), InputBinding.IMPLEMENT_EXTRA4);
		end;
		
		if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
			self:rpmAndAttacher(false, true);
		end;
		if self.realAttacher then
			g_currentMission:addHelpButtonText(g_i18n:getText("noRealAttacher"), InputBinding.LOWER_IMPLEMENT);
		elseif not self.realAttacher then
			g_currentMission:addHelpButtonText(g_i18n:getText("realAttacher"), InputBinding.LOWER_IMPLEMENT);
		end;
	end;

end;

function Allrounder:draw()

	if self:getIsActive() then

		if self.implementAttached ~= nil then
			if self:getRealAnimationTime("lift") >= self:getAnimationDuration("lift") then
				g_currentMission:addHelpButtonText(g_i18n:getText("Detach_Body_swap"), InputBinding.DetachBodyOne);      
			end;
			if 	self.wasToFast then
				g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Speed_Limit_with_Body") .. "\n" .. string.format(InputBinding.getKeyNamesOfDigitalAction(InputBinding.LOWER_IMPLEMENT)), 0.07+0.022, 0.019+0.029));
			end;
			if self.liftVehicle then
				g_currentMission:addWarning(g_i18n:getText("Stop") .. "\n" .. string.format(g_i18n:getText("First_Lift") .. "\n" .. string.format(InputBinding.getKeyNamesOfDigitalAction(InputBinding.LOWER_IMPLEMENT)), 0.07+0.022, 0.019+0.029));
			end;

			if self.implementAttached:getIsFoldAllowed() then
					if self.implementAttached.foldMiddleAnimTime == nil or self.implementAttached.foldMiddleAnimTime ~= 1 then
						if self.implementAttached:getToggledFoldDirection() > 0 then
							g_currentMission:addHelpButtonText(string.format(g_i18n:getText(self.implementAttached.posDirectionText), self.implementAttached.typeDesc), self.implementAttached.foldInputButton);
						else
							g_currentMission:addHelpButtonText(string.format(g_i18n:getText(self.implementAttached.negDirectionText), self.implementAttached.typeDesc), self.implementAttached.foldInputButton);
						end;
					end;
			end;
			
			if self.implementAttached.setIsTurnedOn ~= nil and self.implementAttached.allowsSpraying ~= nil then
				if self.implementAttached.isTurnedOn then
					g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.implementAttached.typeDesc), InputBinding.IMPLEMENT_EXTRA);
				else
					g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.implementAttached.typeDesc), InputBinding.IMPLEMENT_EXTRA);
				end;
			end;
	
		end;
		
		if self.dangerWarning then
			g_currentMission:addWarning(g_i18n:getText("Emergency_function_1") .. "\n"..string.format(tostring(" ").. "\n" .. string.format(g_i18n:getText("Emergency_function_2")), 0.07+0.022, 0.019+0.029));
		end;

		if self.wasToFastNew then
			g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Speed_Limit_with_Lift") .. "\n" .. string.format(InputBinding.getKeyNamesOfDigitalAction(InputBinding.LOWER_IMPLEMENT)), 0.07+0.022, 0.019+0.029));
		end;
		
		if self.implementAttached == nil and self.distance <= 10 then
			g_currentMission:addExtraPrintText(g_i18n:getText("distance_to_swaptrailer") .." "..self.distance);
		end;
		if self.implementAttached == nil and self.firstLower and self.realAttacher then
			setTextBold(true);
			renderText(0.375, 0.95, 0.019, g_i18n:getText("first_must_Lower1"));
			renderText(0.375, 0.93, 0.019, g_i18n:getText("first_must_Lower2"));
			renderText(0.375, 0.91, 0.019, g_i18n:getText("first_must_Lower3"));
		elseif self.implementAttached == nil and self.firstLower and not self.realAttacher then
			renderText(0.375, 0.95, 0.019, g_i18n:getText("first_Lower"));
		end;

	end;

end;

function Allrounder:rpmAndAttacher(moreRpm, realAttacher, noEventSend)
	if not noEventSend then
		RpmAndAttacherEvent.sendEvent(self, moreRpm, realAttacher, noEventSend)
	end;
		if moreRpm then
			self.moreRpm = (not self.moreRpm)
		end;
		
		if realAttacher then
			self.realAttacher = (not self.realAttacher)
		end;

end;

function Allrounder:attacherMoving(attacherMoved, attacherState, noEventSend)
	if not noEventSend then
		AttacherMovingEvent.sendEvent(self, attacherMoved, attacherState, noEventSend)
	end;
		if attacherMoved and attacherState == 0 then
			setTranslation(self.attacherJointToTrans.index, self.attacherJointToTrans.startX, self.attacherJointToTrans.movedY, self.attacherJointToTrans.startZ);
		end;
		
		if attacherMoved and attacherState == 1 then
			setTranslation(self.attacherJointToTrans.index, self.attacherJointToTrans.startX, self.attacherJointToTrans.startY, self.attacherJointToTrans.startZ);
		end;

end;

function Allrounder:capacityChange(changeCapacity, attachedVehicle, noEventSend)
	if not noEventSend then
		CapacityChangeEvent.sendEvent(self, changeCapacity, attachedVehicle, noEventSend)
	end;	
	
	self.implementAttached = attachedVehicle;


		if changeCapacity then
			if self.implementAttached.capacity ~= nil and self.implementAttached.capacity ~= 0 then
				self.capacity = self.implementAttached.capacity;
				self.fillLevel = self.implementAttached.fillLevel;
				self.currentFillType = self.implementAttached.currentFillType;
				self.implementAttached.capacity = 0;
				self.implementAttached.fillLevel = 0;
				self.implementAttached.currentFillType = Fillable.FILLTYPE_UNKNOWN;
				self:setFillLevel(self.fillLevel, self.currentFillType);
				self.implementAttached:setFillLevel(self.implementAttached.fillLevel, self.currentFillType); --Fillable.FILLTYPE_FUEL
			end;
		end;

		if changeCapacity == false then
			if self.capacity ~= 0 then
				self.implementAttached.capacity = self.capacity;
				self.implementAttached.fillLevel = self.fillLevel;
				self.implementAttached.currentFillType = self.currentFillType;
				self.capacity = 0;
				self.currentFillType = Fillable.FILLTYPE_UNKNOWN;
				self.fillLevel = 0;
				self:setFillLevel(self.fillLevel, self.implementAttached.currentFillType);
				self.implementAttached:setFillLevel(self.implementAttached.fillLevel, self.implementAttached.currentFillType);
			end;
		end;
		
	
end;

function Allrounder:newAnimationValues(distanceSmallEnough, distanceLargeEnough, noEventSend)
	if not noEventSend then
		NewAnimationValuesEvent.sendEvent(self, distanceSmallEnough, distanceLargeEnough, noEventSend)
	end;

		if distanceSmallEnough then
		   for i=1, table.getn(self.animationNames) do
		    local name = self.animationNames[i];

			for k=1, table.getn(self.newAnimations[name].newParts) do
				if self.newAnimations[name].newParts[k].startRot ~= nil then
					self.animations[name].parts[k].startRot = self.newAnimations[name].newParts[k].startRot;
				end;
				if self.newAnimations[name].newParts[k].endRot ~= nil then
					self.animations[name].parts[k].endRot = self.newAnimations[name].newParts[k].endRot;
				end;
				if self.newAnimations[name].newParts[k].startTrans ~= nil then
					self.animations[name].parts[k].startTrans = self.newAnimations[name].newParts[k].startTrans;
				end;
				if self.newAnimations[name].newParts[k].endTrans ~= nil then
					self.animations[name].parts[k].endTrans = self.newAnimations[name].newParts[k].endTrans;
				end;
				if self.newAnimations[name].newParts[k].startScale ~= nil then
					self.animations[name].parts[k].startScale = self.newAnimations[name].newParts[k].startScale;
				end;
				if self.newAnimations[name].newParts[k].endScale ~= nil then
					self.animations[name].parts[k].endScale = self.newAnimations[name].newParts[k].endScale;
				end;
			end;
			self.distanceSmallEnough = false;
			self.distanceLargeEnough = true;
			end;
		end;
												
		if distanceLargeEnough then
		   for i=1, table.getn(self.animationNames) do
		    local name = self.animationNames[i];
			for k=1, table.getn(self.newAnimations[name].newParts) do
				if self.newAnimations[name].newParts[k].startRot ~= nil then
					self.animations[name].parts[k].startRot = self.backupAnimations[name].value[k].startRotPos;
				end;
				if self.newAnimations[name].newParts[k].endRot ~= nil then
					self.animations[name].parts[k].endRot = self.backupAnimations[name].value[k].endRotPos;
				end;
				if self.newAnimations[name].newParts[k].startTrans ~= nil then
					self.animations[name].parts[k].startTrans = self.backupAnimations[name].value[k].startTransPos;
				end;
				if self.newAnimations[name].newParts[k].endTrans ~= nil then
					self.animations[name].parts[k].endTrans = self.backupAnimations[name].value[k].endTransPos;
				end;
				if self.newAnimations[name].newParts[k].startScale ~= nil then
					self.animations[name].parts[k].startScale = self.backupAnimations[name].value[k].startScalePos;
				end;
				if self.newAnimations[name].newParts[k].endScale ~= nil then
					self.animations[name].parts[k].endScale = self.backupAnimations[name].value[k].endScalePos;
				end;
			end;
			self.distanceSmallEnough = true;
			self.distanceLargeEnough = false;
			end;
		end;

end;

function Allrounder:lowerAllrounder()

	if self.toFastLower and self.implementAttached ~= nil and self:getRealAnimationTime("lift") >= self:getAnimationDuration("lift") then
		if self:getIsFoldAllowed() then
			self:setFoldState(self:getToggledFoldDirection(), false);
		end;
	end;
	if self.toFastLower2 and self.implementAttached == nil and self:getRealAnimationTime("lift") >= self:getAnimationDuration("lift") then
		if self:getIsFoldAllowed() then
			self:setFoldState(self:getToggledFoldDirection(), false);
		end;
	end;
	
end;

function Allrounder:detachBody(attacherVehicle)
	if self.vehicleAttach ~= nil then
		for k,v in pairs(self.attachedImplements) do
			if v.object.fuelFill ~= nil and v.object.liquidManure == nil then
				self.implementAttached = v.object;
				self:capacityChange(false, v.object);
			end;
			if v.object.detachFromAllrounder ~= nil then
				self.schemaOverlay.overlay = self.newSchemaOverlay.overlay;
				self.schemaOverlay.overlaySelected = self.newSchemaOverlay.overlaySelected;
				v.object:detachFromAllrounder();
				self.vehicleAttach:playDetachSound();
			end;
		end;
		if self.secondAttacher ~= nil then
			self.secondAttacher:playDetachSound();
		end;
	end;

end;

function Allrounder:detachImplement()
	self.implementAttached = nil;
	self.searchImplement = false;
	self.attacherMoved = true;
	self.schemaOverlay.overlay = self.newSchemaOverlay.overlay;
	self.schemaOverlay.overlaySelected = self.newSchemaOverlay.overlaySelected;
	self.loadSavegame = true;
end;

function Allrounder:attachImplement()
	self.searchImplement = true;
	self.firstLower = false;
end;

function Allrounder:onAttach(attacherVehicle)
	self.vehicleAttach = attacherVehicle;
	self.searchImplement = true;
	self.secondAttacher = attacherVehicle.attacherVehicle;
	if self.vehicleAttach.motor ~= nil then
		self.lastRpm = self.vehicleAttach.motor.minRpm;
	end;
	if self.vehicleAttach.motor == nil and self.secondAttacher.motor ~= nil then
		self.lastRpm = self.secondAttacher.motor.minRpm;
	end;

end;

function Allrounder:onDeactivate()

	if self.startTimer then
		self.startTimer = false;
		self.liftVehicle = false;
		self.timerLift = 0;
	end;
		
	if self.startTimerLower then
		self.startTimerLower = false;
		self.dangerWarning = false;
		self.timerDanger = 0;
	end;
		
	if self.vehicleAttach.motor ~= nil then
		self.lastRpm = self.vehicleAttach.motor.minRpm;
	end;
	if self.vehicleAttach.motor == nil and self.secondAttacher.motor ~= nil then
		self.lastRpm = self.secondAttacher.motor.minRpm;
	end;

end;

function Allrounder:onDetach()
	if self.vehicleAttach.motor ~= nil then
		self.vehicleAttach.motor.minRpm = self.lastRpm;
	end;
	if self.vehicleAttach.motor == nil and self.secondAttacher.motor ~= nil then
		self.secondAttacher.motor.minRpm = self.lastRpm;
	end;
	self.loadSavegame = false;
	self.vehicleAttach = nil;
	self.secondAttacher = nil;
	self.startTimer = false;
	self.startTimerLower = false;
	self.liftVehicle = false;
	self.dangerWarning = false;
	self.timerLift = 0;
	self.timerDanger = 0;
	self.lastRpm = 0;
end;

